@@ -1,4 +1,4 @@
-from werkzeug._compat import to_bytes
+from werkzeug._internal import _to_bytes
 from werkzeug.middleware.dispatcher import DispatcherMiddleware
 from werkzeug.test import create_environ
 from werkzeug.test import run_wsgi_app
@@ -11,7 +11,7 @@ def test_dispatcher():
 
     def dummy_application(environ, start_response):
         start_response("200 OK", [("Content-Type", "text/plain")])
-        yield to_bytes(environ["SCRIPT_NAME"])
+        yield _to_bytes(environ["SCRIPT_NAME"])
 
     app = DispatcherMiddleware(
         null_application,
@@ -27,7 +27,7 @@ def test_dispatcher():
             environ = create_environ(p)
             app_iter, status, headers = run_wsgi_app(app, environ)
             assert status == "200 OK"
-            assert b"".join(app_iter).strip() == to_bytes(name)
+            assert b"".join(app_iter).strip() == _to_bytes(name)
 
     app_iter, status, headers = run_wsgi_app(app, create_environ("/missing"))
     assert status == "404 NOT FOUND"
