@@ -1,52 +1,21 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.datastructures
-    ~~~~~~~~~~~~~~~~~~~~
-
-    Tests the functionality of the provided Werkzeug
-    datastructures.
-
-    Classes prefixed with an underscore are mixins and are not discovered by
-    the test runner.
-
-    TODO:
-
-    -   FileMultiDict
-    -   Immutable types undertested
-    -   Split up dict tests
-
-    :copyright: 2007 Pallets
-    :license: BSD-3-Clause
-"""
 import io
 import pickle
 import tempfile
+import typing as t
 from contextlib import contextmanager
 from copy import copy
 from copy import deepcopy
 
 import pytest
 
-from . import strict_eq
-from werkzeug import datastructures
+from werkzeug import datastructures as ds
 from werkzeug import http
-from werkzeug._compat import iteritems
-from werkzeug._compat import iterkeys
-from werkzeug._compat import iterlists
-from werkzeug._compat import iterlistvalues
-from werkzeug._compat import itervalues
-from werkzeug._compat import PY2
-from werkzeug._compat import text_type
-from werkzeug.datastructures import LanguageAccept
-from werkzeug.datastructures import MIMEAccept
-from werkzeug.datastructures import Range
 from werkzeug.exceptions import BadRequestKeyError
 
 
-class TestNativeItermethods(object):
+class TestNativeItermethods:
     def test_basic(self):
-        @datastructures.native_itermethods(["keys", "values", "items"])
-        class StupidDict(object):
+        class StupidDict:
             def keys(self, multi=1):
                 return iter(["a", "b", "c"] * multi)
 
@@ -55,7 +24,7 @@ class TestNativeItermethods(object):
 
             def items(self, multi=1):
                 return iter(
-                    zip(iterkeys(self, multi=multi), itervalues(self, multi=multi))
+                    zip(iter(self.keys(multi=multi)), iter(self.values(multi=multi)))
                 )
 
         d = StupidDict()
@@ -63,17 +32,17 @@ class TestNativeItermethods(object):
         expected_values = [1, 2, 3]
         expected_items = list(zip(expected_keys, expected_values))
 
-        assert list(iterkeys(d)) == expected_keys
-        assert list(itervalues(d)) == expected_values
-        assert list(iteritems(d)) == expected_items
+        assert list(d.keys()) == expected_keys
+        assert list(d.values()) == expected_values
+        assert list(d.items()) == expected_items
 
-        assert list(iterkeys(d, 2)) == expected_keys * 2
-        assert list(itervalues(d, 2)) == expected_values * 2
-        assert list(iteritems(d, 2)) == expected_items * 2
+        assert list(d.keys(2)) == expected_keys * 2
+        assert list(d.values(2)) == expected_values * 2
+        assert list(d.items(2)) == expected_items * 2
 
 
-class _MutableMultiDictTests(object):
-    storage_class = None
+class _MutableMultiDictTests:
+    storage_class: t.Type["ds.MultiDict"]
 
     def test_pickle(self):
         cls = self.storage_class
@@ -101,6 +70,13 @@ class _MutableMultiDictTests(object):
             ud[b"newkey"] = b"bla"
             assert ud != d
 
+    def test_multidict_dict_interop(self):
+        # https://github.com/pallets/werkzeug/pull/2043
+        md = self.storage_class([("a", 1), ("a", 2)])
+        assert dict(md)["a"] != [1, 2]
+        assert dict(md)["a"] == 1
+        assert dict(md) == {**md} == {"a": 1}
+
     def test_basic_interface(self):
         md = self.storage_class()
         assert isinstance(md, dict)
@@ -161,10 +137,10 @@ class _MutableMultiDictTests(object):
 
         # keys, values, items, lists
         assert list(sorted(md.keys())) == ["a", "b", "c"]
-        assert list(sorted(iterkeys(md))) == ["a", "b", "c"]
+        assert list(sorted(md.keys())) == ["a", "b", "c"]
 
-        assert list(sorted(itervalues(md))) == [1, 2, 3]
-        assert list(sorted(itervalues(md))) == [1, 2, 3]
+        assert list(sorted(md.values())) == [1, 2, 3]
+        assert list(sorted(md.values())) == [1, 2, 3]
 
         assert list(sorted(md.items())) == [("a", 1), ("b", 2), ("c", 3)]
         assert list(sorted(md.items(multi=True))) == [
@@ -174,8 +150,8 @@ class _MutableMultiDictTests(object):
             ("b", 2),
             ("c", 3),
         ]
-        assert list(sorted(iteritems(md))) == [("a", 1), ("b", 2), ("c", 3)]
-        assert list(sorted(iteritems(md, multi=True))) == [
+        assert list(sorted(md.items())) == [("a", 1), ("b", 2), ("c", 3)]
+        assert list(sorted(md.items(multi=True))) == [
             ("a", 1),
             ("a", 2),
             ("a", 3),
@@ -184,7 +160,7 @@ class _MutableMultiDictTests(object):
         ]
 
         assert list(sorted(md.lists())) == [("a", [1, 2, 3]), ("b", [2]), ("c", [3])]
-        assert list(sorted(iterlists(md))) == [("a", [1, 2, 3]), ("b", [2]), ("c", [3])]
+        assert list(sorted(md.lists())) == [("a", [1, 2, 3]), ("b", [2]), ("c", [3])]
 
         # copy method
         c = md.copy()
@@ -282,8 +258,8 @@ class _MutableMultiDictTests(object):
         assert md.getlist("foo") == [1, 2]
 
 
-class _ImmutableDictTests(object):
-    storage_class = None
+class _ImmutableDictTests:
+    storage_class: t.Type[dict]
 
     def test_follows_dict_interface(self):
         cls = self.storage_class
@@ -315,7 +291,7 @@ class _ImmutableDictTests(object):
         cls = self.storage_class
         immutable = cls({"a": 1, "b": 2})
         immutable2 = cls({"a": 2, "b": 2})
-        x = set([immutable])
+        x = {immutable}
         assert immutable in x
         assert immutable2 not in x
         x.discard(immutable)
@@ -330,17 +306,17 @@ class _ImmutableDictTests(object):
 
 
 class TestImmutableTypeConversionDict(_ImmutableDictTests):
-    storage_class = datastructures.ImmutableTypeConversionDict
+    storage_class = ds.ImmutableTypeConversionDict
 
 
 class TestImmutableMultiDict(_ImmutableDictTests):
-    storage_class = datastructures.ImmutableMultiDict
+    storage_class = ds.ImmutableMultiDict
 
     def test_multidict_is_hashable(self):
         cls = self.storage_class
         immutable = cls({"a": [1, 2], "b": 2})
         immutable2 = cls({"a": [1], "b": 2})
-        x = set([immutable])
+        x = {immutable}
         assert immutable in x
         assert immutable2 not in x
         x.discard(immutable)
@@ -355,11 +331,11 @@ class TestImmutableMultiDict(_ImmutableDictTests):
 
 
 class TestImmutableDict(_ImmutableDictTests):
-    storage_class = datastructures.ImmutableDict
+    storage_class = ds.ImmutableDict
 
 
 class TestImmutableOrderedMultiDict(_ImmutableDictTests):
-    storage_class = datastructures.ImmutableOrderedMultiDict
+    storage_class = ds.ImmutableOrderedMultiDict
 
     def test_ordered_multidict_is_hashable(self):
         a = self.storage_class([("a", 1), ("b", 1), ("a", 2)])
@@ -368,7 +344,7 @@ class TestImmutableOrderedMultiDict(_ImmutableDictTests):
 
 
 class TestMultiDict(_MutableMultiDictTests):
-    storage_class = datastructures.MultiDict
+    storage_class = ds.MultiDict
 
     def test_multidict_pop(self):
         def make_d():
@@ -424,40 +400,8 @@ class TestMultiDict(_MutableMultiDictTests):
         ]
         md = self.storage_class(mapping)
         assert list(zip(md.keys(), md.listvalues())) == list(md.lists())
-        assert list(zip(md, iterlistvalues(md))) == list(iterlists(md))
-        assert list(zip(iterkeys(md), iterlistvalues(md))) == list(iterlists(md))
-
-    @pytest.mark.skipif(not PY2, reason="viewmethods work only for the 2-nd version.")
-    def test_view_methods(self):
-        mapping = [("a", "b"), ("a", "c")]
-        md = self.storage_class(mapping)
-
-        vi = md.viewitems()  # noqa: B302
-        vk = md.viewkeys()  # noqa: B302
-        vv = md.viewvalues()  # noqa: B302
-
-        assert list(vi) == list(md.items())
-        assert list(vk) == list(md.keys())
-        assert list(vv) == list(md.values())
-
-        md["k"] = "n"
-
-        assert list(vi) == list(md.items())
-        assert list(vk) == list(md.keys())
-        assert list(vv) == list(md.values())
-
-    @pytest.mark.skipif(not PY2, reason="viewmethods work only for the 2-nd version.")
-    def test_viewitems_with_multi(self):
-        mapping = [("a", "b"), ("a", "c")]
-        md = self.storage_class(mapping)
-
-        vi = md.viewitems(multi=True)  # noqa: B302
-
-        assert list(vi) == list(md.items(multi=True))
-
-        md["k"] = "n"
-
-        assert list(vi) == list(md.items(multi=True))
+        assert list(zip(md, md.listvalues())) == list(md.lists())
+        assert list(zip(md.keys(), md.listvalues())) == list(md.lists())
 
     def test_getitem_raise_badrequestkeyerror_for_empty_list_value(self):
         mapping = [("a", "b"), ("a", "c")]
@@ -470,7 +414,7 @@ class TestMultiDict(_MutableMultiDictTests):
 
 
 class TestOrderedMultiDict(_MutableMultiDictTests):
-    storage_class = datastructures.OrderedMultiDict
+    storage_class = ds.OrderedMultiDict
 
     def test_ordered_interface(self):
         cls = self.storage_class
@@ -481,9 +425,9 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         assert len(d) == 1
         d.add("foo", "baz")
         assert len(d) == 1
-        assert list(iteritems(d)) == [("foo", "bar")]
+        assert list(d.items()) == [("foo", "bar")]
         assert list(d) == ["foo"]
-        assert list(iteritems(d, multi=True)) == [("foo", "bar"), ("foo", "baz")]
+        assert list(d.items(multi=True)) == [("foo", "bar"), ("foo", "baz")]
         del d["foo"]
         assert not d
         assert len(d) == 0
@@ -493,15 +437,15 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         d.add("foo", 3)
         assert d.getlist("foo") == [1, 2, 3]
         assert d.getlist("bar") == [42]
-        assert list(iteritems(d)) == [("foo", 1), ("bar", 42)]
+        assert list(d.items()) == [("foo", 1), ("bar", 42)]
 
         expected = ["foo", "bar"]
 
         assert list(d.keys()) == expected
         assert list(d) == expected
-        assert list(iterkeys(d)) == expected
+        assert list(d.keys()) == expected
 
-        assert list(iteritems(d, multi=True)) == [
+        assert list(d.items(multi=True)) == [
             ("foo", 1),
             ("foo", 2),
             ("bar", 42),
@@ -516,13 +460,13 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         assert d.poplist("bar") == [42]
         assert not d
 
-        d.get("missingkey") is None
+        assert d.get("missingkey") is None
 
         d.add("foo", 42)
         d.add("foo", 23)
         d.add("bar", 2)
         d.add("foo", 42)
-        assert d == datastructures.MultiDict(d)
+        assert d == ds.MultiDict(d)
         id = self.storage_class(d)
         assert d == id
         d.add("foo", 2)
@@ -567,20 +511,20 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         pytest.raises(TypeError, hash, d)
 
     def test_iterables(self):
-        a = datastructures.MultiDict((("key_a", "value_a"),))
-        b = datastructures.MultiDict((("key_b", "value_b"),))
-        ab = datastructures.CombinedMultiDict((a, b))
+        a = ds.MultiDict((("key_a", "value_a"),))
+        b = ds.MultiDict((("key_b", "value_b"),))
+        ab = ds.CombinedMultiDict((a, b))
 
         assert sorted(ab.lists()) == [("key_a", ["value_a"]), ("key_b", ["value_b"])]
         assert sorted(ab.listvalues()) == [["value_a"], ["value_b"]]
         assert sorted(ab.keys()) == ["key_a", "key_b"]
 
-        assert sorted(iterlists(ab)) == [("key_a", ["value_a"]), ("key_b", ["value_b"])]
-        assert sorted(iterlistvalues(ab)) == [["value_a"], ["value_b"]]
-        assert sorted(iterkeys(ab)) == ["key_a", "key_b"]
+        assert sorted(ab.lists()) == [("key_a", ["value_a"]), ("key_b", ["value_b"])]
+        assert sorted(ab.listvalues()) == [["value_a"], ["value_b"]]
+        assert sorted(ab.keys()) == ["key_a", "key_b"]
 
     def test_get_description(self):
-        data = datastructures.OrderedMultiDict()
+        data = ds.OrderedMultiDict()
 
         with pytest.raises(BadRequestKeyError) as exc_info:
             data["baz"]
@@ -598,8 +542,8 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         assert "baz" not in exc_info.value.get_description()
 
 
-class TestTypeConversionDict(object):
-    storage_class = datastructures.TypeConversionDict
+class TestTypeConversionDict:
+    storage_class = ds.TypeConversionDict
 
     def test_value_conversion(self):
         d = self.storage_class(foo="1")
@@ -616,12 +560,12 @@ class TestTypeConversionDict(object):
             d.get("foo", type=lambda x: switch[x])
 
 
-class TestCombinedMultiDict(object):
-    storage_class = datastructures.CombinedMultiDict
+class TestCombinedMultiDict:
+    storage_class = ds.CombinedMultiDict
 
     def test_basic_interface(self):
-        d1 = datastructures.MultiDict([("foo", "1")])
-        d2 = datastructures.MultiDict([("bar", "2"), ("bar", "3")])
+        d1 = ds.MultiDict([("foo", "1")])
+        d2 = ds.MultiDict([("bar", "2"), ("bar", "3")])
         d = self.storage_class([d1, d2])
 
         # lookup
@@ -651,14 +595,18 @@ class TestCombinedMultiDict(object):
         d["foo"] = "blub"
 
         # make sure lists merges
-        md1 = datastructures.MultiDict((("foo", "bar"),))
-        md2 = datastructures.MultiDict((("foo", "blafasel"),))
+        md1 = ds.MultiDict((("foo", "bar"), ("foo", "baz")))
+        md2 = ds.MultiDict((("foo", "blafasel"),))
         x = self.storage_class((md1, md2))
-        assert list(iterlists(x)) == [("foo", ["bar", "blafasel"])]
+        assert list(x.lists()) == [("foo", ["bar", "baz", "blafasel"])]
+
+        # make sure dicts are created properly
+        assert x.to_dict() == {"foo": "bar"}
+        assert x.to_dict(flat=False) == {"foo": ["bar", "baz", "blafasel"]}
 
     def test_length(self):
-        d1 = datastructures.MultiDict([("foo", "1")])
-        d2 = datastructures.MultiDict([("bar", "2")])
+        d1 = ds.MultiDict([("foo", "1")])
+        d2 = ds.MultiDict([("bar", "2")])
         assert len(d1) == len(d2) == 1
         d = self.storage_class([d1, d2])
         assert len(d) == 2
@@ -667,8 +615,8 @@ class TestCombinedMultiDict(object):
         assert len(d) == 1
 
 
-class TestHeaders(object):
-    storage_class = datastructures.Headers
+class TestHeaders:
+    storage_class = ds.Headers
 
     def test_basic_interface(self):
         headers = self.storage_class()
@@ -767,7 +715,7 @@ class TestHeaders(object):
         h.set("X-Foo-Poo", "bleh")
         h.set("Content-Type", "application/whocares")
         h.set("X-Forwarded-For", "192.168.0.123")
-        h[:] = [(k, v) for k, v in h if k.startswith(u"X-")]
+        h[:] = [(k, v) for k, v in h if k.startswith("X-")]
         assert list(h) == [("X-Foo-Poo", "bleh"), ("X-Forwarded-For", "192.168.0.123")]
 
     def test_bytes_operations(self):
@@ -782,7 +730,7 @@ class TestHeaders(object):
 
     def test_extend(self):
         h = self.storage_class([("a", "0"), ("b", "1"), ("c", "2")])
-        h.extend(datastructures.Headers([("a", "3"), ("a", "4")]))
+        h.extend(ds.Headers([("a", "3"), ("a", "4")]))
         assert h.getlist("a") == ["0", "3", "4"]
         h.extend(b=["5", "6"])
         assert h.getlist("b") == ["1", "5", "6"]
@@ -795,7 +743,7 @@ class TestHeaders(object):
 
     def test_update(self):
         h = self.storage_class([("a", "0"), ("b", "1"), ("c", "2")])
-        h.update(datastructures.Headers([("a", "3"), ("a", "4")]))
+        h.update(ds.Headers([("a", "3"), ("a", "4")]))
         assert h.getlist("a") == ["3", "4"]
         h.update(b=["5", "6"])
         assert h.getlist("b") == ["5", "6"]
@@ -825,25 +773,17 @@ class TestHeaders(object):
 
     def test_to_wsgi_list(self):
         h = self.storage_class()
-        h.set(u"Key", u"Value")
+        h.set("Key", "Value")
         for key, value in h.to_wsgi_list():
-            if PY2:
-                strict_eq(key, b"Key")
-                strict_eq(value, b"Value")
-            else:
-                strict_eq(key, u"Key")
-                strict_eq(value, u"Value")
+            assert key == "Key"
+            assert value == "Value"
 
     def test_to_wsgi_list_bytes(self):
         h = self.storage_class()
         h.set(b"Key", b"Value")
         for key, value in h.to_wsgi_list():
-            if PY2:
-                strict_eq(key, b"Key")
-                strict_eq(value, b"Value")
-            else:
-                strict_eq(key, u"Key")
-                strict_eq(value, u"Value")
+            assert key == "Key"
+            assert value == "Value"
 
     def test_equality(self):
         # test equality, given keys are case insensitive
@@ -860,8 +800,8 @@ class TestHeaders(object):
         assert h1 == h2
 
 
-class TestEnvironHeaders(object):
-    storage_class = datastructures.EnvironHeaders
+class TestEnvironHeaders:
+    storage_class = ds.EnvironHeaders
 
     def test_basic_interface(self):
         # this happens in multiple WSGI servers because they
@@ -895,29 +835,21 @@ class TestEnvironHeaders(object):
         headers = self.storage_class(env)
         assert dict(headers) == {"X-Foo": "42", "Content-Length": "0"}
 
-    def test_return_type_is_unicode(self):
-        # environ contains native strings; we return unicode
-        headers = self.storage_class(
-            {"HTTP_FOO": "\xe2\x9c\x93", "CONTENT_TYPE": "text/plain"}
-        )
-        assert headers["Foo"] == u"\xe2\x9c\x93"
-        assert isinstance(headers["Foo"], text_type)
-        assert isinstance(headers["Content-Type"], text_type)
-        iter_output = dict(iter(headers))
-        assert iter_output["Foo"] == u"\xe2\x9c\x93"
-        assert isinstance(iter_output["Foo"], text_type)
-        assert isinstance(iter_output["Content-Type"], text_type)
+    def test_return_type_is_str(self):
+        headers = self.storage_class({"HTTP_FOO": "\xe2\x9c\x93"})
+        assert headers["Foo"] == "\xe2\x9c\x93"
+        assert next(iter(headers)) == ("Foo", "\xe2\x9c\x93")
 
     def test_bytes_operations(self):
         foo_val = "\xff"
         h = self.storage_class({"HTTP_X_FOO": foo_val})
 
         assert h.get("x-foo", as_bytes=True) == b"\xff"
-        assert h.get("x-foo") == u"\xff"
+        assert h.get("x-foo") == "\xff"
 
 
-class TestHeaderSet(object):
-    storage_class = datastructures.HeaderSet
+class TestHeaderSet:
+    storage_class = ds.HeaderSet
 
     def test_basic_interface(self):
         hs = self.storage_class()
@@ -941,8 +873,8 @@ class TestHeaderSet(object):
         assert not hs
 
 
-class TestImmutableList(object):
-    storage_class = datastructures.ImmutableList
+class TestImmutableList:
+    storage_class = ds.ImmutableList
 
     def test_list_hashable(self):
         data = (1, 2, 3, 4)
@@ -978,8 +910,8 @@ def make_call_asserter(func=None):
     return asserter, wrapped
 
 
-class TestCallbackDict(object):
-    storage_class = datastructures.CallbackDict
+class TestCallbackDict:
+    storage_class = ds.CallbackDict
 
     def test_callback_dict_reads(self):
         assert_calls, func = make_call_asserter()
@@ -990,7 +922,7 @@ class TestCallbackDict(object):
             dct["a"]
             dct.get("a")
             pytest.raises(KeyError, lambda: dct["x"])
-            "a" in dct
+            assert "a" in dct
             list(iter(dct))
             dct.copy()
         with assert_calls(0, "callback triggered without modification"):
@@ -1018,23 +950,21 @@ class TestCallbackDict(object):
             pytest.raises(KeyError, lambda: dct.pop("x"))
 
 
-class TestCacheControl(object):
+class TestCacheControl:
     def test_repr(self):
-        cc = datastructures.RequestCacheControl([("max-age", "0"), ("private", "True")])
+        cc = ds.RequestCacheControl([("max-age", "0"), ("private", "True")])
         assert repr(cc) == "<RequestCacheControl max-age='0' private='True'>"
 
     def test_set_none(self):
-        cc = datastructures.ResponseCacheControl([("max-age", "0")])
+        cc = ds.ResponseCacheControl([("max-age", "0")])
         assert cc.no_cache is None
         cc.no_cache = None
         assert cc.no_cache is None
 
 
-class TestContentSecurityPolicy(object):
+class TestContentSecurityPolicy:
     def test_construct(self):
-        csp = datastructures.ContentSecurityPolicy(
-            [("font-src", "'self'"), ("media-src", "*")]
-        )
+        csp = ds.ContentSecurityPolicy([("font-src", "'self'"), ("media-src", "*")])
         assert csp.font_src == "'self'"
         assert csp.media_src == "*"
         policies = [policy.strip() for policy in csp.to_header().split(";")]
@@ -1042,7 +972,7 @@ class TestContentSecurityPolicy(object):
         assert "media-src *" in policies
 
     def test_properties(self):
-        csp = datastructures.ContentSecurityPolicy()
+        csp = ds.ContentSecurityPolicy()
         csp.default_src = "* 'self' quart.com"
         csp.img_src = "'none'"
         policies = [policy.strip() for policy in csp.to_header().split(";")]
@@ -1050,8 +980,8 @@ class TestContentSecurityPolicy(object):
         assert "img-src 'none'" in policies
 
 
-class TestAccept(object):
-    storage_class = datastructures.Accept
+class TestAccept:
+    storage_class = ds.Accept
 
     def test_accept_basic(self):
         accept = self.storage_class(
@@ -1131,7 +1061,7 @@ class TestAccept(object):
         assert accept.best == "a"
 
 
-class TestMIMEAccept(object):
+class TestMIMEAccept:
     @pytest.mark.parametrize(
         ("values", "matches", "default", "expect"),
         [
@@ -1171,12 +1101,12 @@ class TestMIMEAccept(object):
         ],
     )
     def test_mime_accept(self, values, matches, default, expect):
-        accept = MIMEAccept(values)
+        accept = ds.MIMEAccept(values)
         match = accept.best_match(matches, default=default)
         assert match == expect
 
 
-class TestLanguageAccept(object):
+class TestLanguageAccept:
     @pytest.mark.parametrize(
         ("values", "matches", "default", "expect"),
         (
@@ -1195,32 +1125,29 @@ class TestLanguageAccept(object):
         ),
     )
     def test_best_match_fallback(self, values, matches, default, expect):
-        accept = LanguageAccept(values)
+        accept = ds.LanguageAccept(values)
         best = accept.best_match(matches, default=default)
         assert best == expect
 
 
-class TestFileStorage(object):
-    storage_class = datastructures.FileStorage
+class TestFileStorage:
+    storage_class = ds.FileStorage
 
     def test_mimetype_always_lowercase(self):
         file_storage = self.storage_class(content_type="APPLICATION/JSON")
         assert file_storage.mimetype == "application/json"
 
-    def test_bytes_proper_sentinel(self):
-        # ensure we iterate over new lines and don't enter into an infinite loop
-        import io
+    @pytest.mark.parametrize("data", [io.StringIO("one\ntwo"), io.BytesIO(b"one\ntwo")])
+    def test_bytes_proper_sentinel(self, data):
+        # iterate over new lines and don't enter an infinite loop
+        storage = self.storage_class(data)
+        idx = -1
 
-        unicode_storage = self.storage_class(io.StringIO(u"one\ntwo"))
-        for idx, _line in enumerate(unicode_storage):
-            assert idx < 2
-        assert idx == 1
-        binary_storage = self.storage_class(io.BytesIO(b"one\ntwo"))
-        for idx, _line in enumerate(binary_storage):
+        for idx, _line in enumerate(storage):
             assert idx < 2
+
         assert idx == 1
 
-    @pytest.mark.skipif(PY2, reason="io.IOBase is only needed in PY3.")
     @pytest.mark.parametrize("stream", (tempfile.SpooledTemporaryFile, io.BytesIO))
     def test_proxy_can_access_stream_attrs(self, stream):
         """``SpooledTemporaryFile`` doesn't implement some of
@@ -1235,9 +1162,10 @@ class TestFileStorage(object):
         for name in ("fileno", "writable", "readable", "seekable"):
             assert hasattr(file_storage, name)
 
+    @pytest.mark.filterwarnings("ignore::pytest.PytestUnraisableExceptionWarning")
     def test_save_to_pathlib_dst(self, tmp_path):
         src = tmp_path / "src.txt"
-        src.write_text(u"test")
+        src.write_text("test")
         storage = self.storage_class(src.open("rb"))
         dst = tmp_path / "dst.txt"
         storage.save(dst)
@@ -1260,7 +1188,7 @@ class TestFileStorage(object):
 
 @pytest.mark.parametrize("ranges", ([(0, 1), (-5, None)], [(5, None)]))
 def test_range_to_header(ranges):
-    header = Range("byes", ranges).to_header()
+    header = ds.Range("byes", ranges).to_header()
     r = http.parse_range_header(header)
     assert r.ranges == ranges
 
@@ -1270,4 +1198,4 @@ def test_range_to_header(ranges):
 )
 def test_range_validates_ranges(ranges):
     with pytest.raises(ValueError):
-        datastructures.Range("bytes", ranges)
+        ds.Range("bytes", ranges)
