@@ -1,19 +1,8 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.routing
-    ~~~~~~~~~~~~~
-
-    Routing tests.
-
-    :copyright: 2007 Pallets
-    :license: BSD-3-Clause
-"""
 import gc
 import uuid
 
 import pytest
 
-from . import strict_eq
 from werkzeug import routing as r
 from werkzeug.datastructures import ImmutableDict
 from werkzeug.datastructures import MultiDict
@@ -73,6 +62,28 @@ def test_basic_routing():
     with pytest.raises(r.WebsocketMismatch):
         adapter.match("/foo", websocket=True)
 
+    adapter = map.bind_to_environ(
+        create_environ(
+            "/ws?foo=bar",
+            "http://example.org/",
+            headers=[("Connection", "Upgrade"), ("upgrade", "WebSocket")],
+        )
+    )
+    assert adapter.match("/ws") == ("ws", {})
+    with pytest.raises(r.WebsocketMismatch):
+        adapter.match("/ws", websocket=False)
+
+    adapter = map.bind_to_environ(
+        create_environ(
+            "/ws?foo=bar",
+            "http://example.org/",
+            headers=[("Connection", "keep-alive, Upgrade"), ("upgrade", "websocket")],
+        )
+    )
+    assert adapter.match("/ws") == ("ws", {})
+    with pytest.raises(r.WebsocketMismatch):
+        adapter.match("/ws", websocket=False)
+
 
 def test_merge_slashes_match():
     url_map = r.Map(
@@ -111,6 +122,26 @@ def test_merge_slashes_match():
     assert adapter.match("/no//merge")[0] == "no_merge"
 
 
+@pytest.mark.parametrize(
+    ("path", "expected"),
+    [("/merge/%//path", "/merge/%25/path"), ("/merge//st/path", "/merge/st/path")],
+)
+def test_merge_slash_encoding(path, expected):
+    """This test is to make sure URLs are not double-encoded
+    when a redirect is thrown with `merge_slash = True`"""
+    url_map = r.Map(
+        [
+            r.Rule("/merge/<some>/path"),
+        ]
+    )
+    adapter = url_map.bind("localhost", "/")
+
+    with pytest.raises(r.RequestRedirect) as excinfo:
+        adapter.match(path)
+
+    assert excinfo.value.new_url.endswith(expected)
+
+
 def test_merge_slashes_build():
     url_map = r.Map(
         [
@@ -176,22 +207,22 @@ def test_strict_slashes_redirect():
 
 def test_environ_defaults():
     environ = create_environ("/foo")
-    strict_eq(environ["PATH_INFO"], "/foo")
+    assert environ["PATH_INFO"] == "/foo"
     m = r.Map([r.Rule("/foo", endpoint="foo"), r.Rule("/bar", endpoint="bar")])
     a = m.bind_to_environ(environ)
-    strict_eq(a.match("/foo"), ("foo", {}))
-    strict_eq(a.match(), ("foo", {}))
-    strict_eq(a.match("/bar"), ("bar", {}))
+    assert a.match("/foo") == ("foo", {})
+    assert a.match() == ("foo", {})
+    assert a.match("/bar") == ("bar", {})
     pytest.raises(r.NotFound, a.match, "/bars")
 
 
 def test_environ_nonascii_pathinfo():
-    environ = create_environ(u"/лошадь")
-    m = r.Map([r.Rule(u"/", endpoint="index"), r.Rule(u"/лошадь", endpoint="horse")])
+    environ = create_environ("/лошадь")
+    m = r.Map([r.Rule("/", endpoint="index"), r.Rule("/лошадь", endpoint="horse")])
     a = m.bind_to_environ(environ)
-    strict_eq(a.match(u"/"), ("index", {}))
-    strict_eq(a.match(u"/лошадь"), ("horse", {}))
-    pytest.raises(r.NotFound, a.match, u"/барсук")
+    assert a.match("/") == ("index", {})
+    assert a.match("/лошадь") == ("horse", {})
+    pytest.raises(r.NotFound, a.match, "/барсук")
 
 
 def test_basic_building():
@@ -235,29 +266,33 @@ def test_basic_building():
     adapter = map.bind("example.org", url_scheme="")
     assert adapter.build("foo", {}) == "/foo"
     assert adapter.build("foo", {}, force_external=True) == "//example.org/foo"
+    assert (
+        adapter.build("foo", {}, url_scheme="https", force_external=True)
+        == "https://example.org/foo"
+    )
 
     adapter = map.bind("example.org", url_scheme="ws")
     assert adapter.build("ws", {}) == "ws://example.org/ws"
     assert adapter.build("foo", {}, force_external=True) == "http://example.org/foo"
     assert adapter.build("foo", {}) == "/foo"
+    assert adapter.build("ws", {}, url_scheme="https") == "wss://example.org/ws"
 
 
 def test_long_build():
-    long_args = dict(("v%d" % x, x) for x in range(10000))
+    long_args = {f"v{x}": x for x in range(10000)}
     map = r.Map(
         [
             r.Rule(
-                "".join("/<%s>" % k for k in long_args.keys()),
+                "".join(f"/<{k}>" for k in long_args.keys()),
                 endpoint="bleep",
                 build_only=True,
             )
         ]
     )
     adapter = map.bind("localhost", "/")
-    url = adapter.build("bleep", long_args)
-    url += "/"
+    url = f"{adapter.build('bleep', long_args)}/"
     for v in long_args.values():
-        assert "/%d" % v in url
+        assert f"/{v}" in url
 
 
 def test_defaults():
@@ -458,34 +493,34 @@ def test_adapter_url_parameter_sorting():
 
 
 def test_request_direct_charset_bug():
-    map = r.Map([r.Rule(u"/öäü/")])
+    map = r.Map([r.Rule("/öäü/")])
     adapter = map.bind("localhost", "/")
 
     with pytest.raises(r.RequestRedirect) as excinfo:
-        adapter.match(u"/öäü")
+        adapter.match("/öäü")
     assert excinfo.value.new_url == "http://localhost/%C3%B6%C3%A4%C3%BC/"
 
 
 def test_request_redirect_default():
-    map = r.Map([r.Rule(u"/foo", defaults={"bar": 42}), r.Rule(u"/foo/<int:bar>")])
+    map = r.Map([r.Rule("/foo", defaults={"bar": 42}), r.Rule("/foo/<int:bar>")])
     adapter = map.bind("localhost", "/")
 
     with pytest.raises(r.RequestRedirect) as excinfo:
-        adapter.match(u"/foo/42")
+        adapter.match("/foo/42")
     assert excinfo.value.new_url == "http://localhost/foo"
 
 
 def test_request_redirect_default_subdomain():
     map = r.Map(
         [
-            r.Rule(u"/foo", defaults={"bar": 42}, subdomain="test"),
-            r.Rule(u"/foo/<int:bar>", subdomain="other"),
+            r.Rule("/foo", defaults={"bar": 42}, subdomain="test"),
+            r.Rule("/foo/<int:bar>", subdomain="other"),
         ]
     )
     adapter = map.bind("localhost", "/", subdomain="other")
 
     with pytest.raises(r.RequestRedirect) as excinfo:
-        adapter.match(u"/foo/42")
+        adapter.match("/foo/42")
     assert excinfo.value.new_url == "http://test.localhost/foo"
 
 
@@ -502,15 +537,15 @@ def test_server_name_interpolation():
         [r.Rule("/", endpoint="index"), r.Rule("/", endpoint="alt", subdomain="alt")]
     )
 
-    env = create_environ("/", "http://%s/" % server_name)
+    env = create_environ("/", f"http://{server_name}/")
     adapter = map.bind_to_environ(env, server_name=server_name)
     assert adapter.match() == ("index", {})
 
-    env = create_environ("/", "http://alt.%s/" % server_name)
+    env = create_environ("/", f"http://alt.{server_name}/")
     adapter = map.bind_to_environ(env, server_name=server_name)
     assert adapter.match() == ("alt", {})
 
-    env = create_environ("/", "http://%s/" % server_name)
+    env = create_environ("/", f"http://{server_name}/")
 
     with pytest.warns(UserWarning):
         adapter = map.bind_to_environ(env, server_name="foo")
@@ -567,7 +602,7 @@ def test_rule_templates():
         ]
     )
 
-    out = sorted([(x.rule, x.subdomain, x.endpoint) for x in url_map.iter_rules()])
+    out = sorted((x.rule, x.subdomain, x.endpoint) for x in url_map.iter_rules())
 
     assert out == [
         ("/blah", "test1", "x_bar"),
@@ -685,7 +720,7 @@ def test_converter_with_tuples():
 
     class TwoValueConverter(r.BaseConverter):
         def __init__(self, *args, **kwargs):
-            super(TwoValueConverter, self).__init__(*args, **kwargs)
+            super().__init__(*args, **kwargs)
             self.regex = r"(\w\w+)/(\w\w+)"
 
         def to_python(self, two_values):
@@ -693,7 +728,7 @@ def test_converter_with_tuples():
             return one, two
 
         def to_url(self, values):
-            return "%s/%s" % (values[0], values[1])
+            return f"{values[0]}/{values[1]}"
 
     map = r.Map(
         [r.Rule("/<two:foo>/", endpoint="handler")],
@@ -718,16 +753,30 @@ def test_anyconverter():
     assert a.match("/a.2") == ("yes_dot", {"a": "a.2"})
 
 
-def test_build_append_unknown():
-    map = r.Map([r.Rule("/bar/<float:bazf>", endpoint="barf")])
-    adapter = map.bind("example.org", "/", subdomain="blah")
+@pytest.mark.parametrize(
+    ("value", "expect"),
+    [
+        (None, ""),
+        ([None], ""),
+        ([None, None], ""),
+        ("", "?extra="),
+        ([""], "?extra="),
+        (1.0, "?extra=1.0"),
+        ([1, 2], "?extra=1&extra=2"),
+        ([1, None, 2], "?extra=1&extra=2"),
+        ([1, "", 2], "?extra=1&extra=&extra=2"),
+    ],
+)
+def test_build_append_unknown(value, expect):
+    map = r.Map([r.Rule("/foo/<name>", endpoint="foo")])
+    adapter = map.bind("example.org", "/", subdomain="test")
     assert (
-        adapter.build("barf", {"bazf": 0.815, "bif": 1.0})
-        == "http://example.org/bar/0.815?bif=1.0"
+        adapter.build("foo", {"name": "bar", "extra": value})
+        == f"http://example.org/foo/bar{expect}"
     )
     assert (
-        adapter.build("barf", {"bazf": 0.815, "bif": 1.0}, append_unknown=False)
-        == "http://example.org/bar/0.815"
+        adapter.build("foo", {"name": "bar", "extra": value}, append_unknown=False)
+        == "http://example.org/foo/bar"
     )
 
 
@@ -840,7 +889,7 @@ def test_external_building_with_port_bind_to_environ_wrong_servername():
 
 
 def test_converter_parser():
-    args, kwargs = r.parse_converter_args(u"test, a=1, b=3.0")
+    args, kwargs = r.parse_converter_args("test, a=1, b=3.0")
 
     assert args == ("test",)
     assert kwargs == {"a": 1, "b": 3.0}
@@ -855,7 +904,7 @@ def test_converter_parser():
     args, kwargs = r.parse_converter_args("True, False, None")
     assert args == (True, False, None)
 
-    args, kwargs = r.parse_converter_args('"foo", u"bar"')
+    args, kwargs = r.parse_converter_args('"foo", "bar"')
     assert args == ("foo", "bar")
 
 
@@ -877,7 +926,7 @@ def test_alias_redirects():
     def ensure_redirect(path, new_url, args=None):
         with pytest.raises(r.RequestRedirect) as excinfo:
             a.match(path, query_args=args)
-        assert excinfo.value.new_url == "http://example.com" + new_url
+        assert excinfo.value.new_url == f"http://example.com{new_url}"
 
     ensure_redirect("/index.html", "/")
     ensure_redirect("/users/index.html", "/users/")
@@ -894,25 +943,25 @@ def test_alias_redirects():
 def test_double_defaults(prefix):
     m = r.Map(
         [
-            r.Rule(prefix + "/", defaults={"foo": 1, "bar": False}, endpoint="x"),
-            r.Rule(prefix + "/<int:foo>", defaults={"bar": False}, endpoint="x"),
-            r.Rule(prefix + "/bar/", defaults={"foo": 1, "bar": True}, endpoint="x"),
-            r.Rule(prefix + "/bar/<int:foo>", defaults={"bar": True}, endpoint="x"),
+            r.Rule(f"{prefix}/", defaults={"foo": 1, "bar": False}, endpoint="x"),
+            r.Rule(f"{prefix}/<int:foo>", defaults={"bar": False}, endpoint="x"),
+            r.Rule(f"{prefix}/bar/", defaults={"foo": 1, "bar": True}, endpoint="x"),
+            r.Rule(f"{prefix}/bar/<int:foo>", defaults={"bar": True}, endpoint="x"),
         ]
     )
     a = m.bind("example.com")
 
-    assert a.match(prefix + "/") == ("x", {"foo": 1, "bar": False})
-    assert a.match(prefix + "/2") == ("x", {"foo": 2, "bar": False})
-    assert a.match(prefix + "/bar/") == ("x", {"foo": 1, "bar": True})
-    assert a.match(prefix + "/bar/2") == ("x", {"foo": 2, "bar": True})
+    assert a.match(f"{prefix}/") == ("x", {"foo": 1, "bar": False})
+    assert a.match(f"{prefix}/2") == ("x", {"foo": 2, "bar": False})
+    assert a.match(f"{prefix}/bar/") == ("x", {"foo": 1, "bar": True})
+    assert a.match(f"{prefix}/bar/2") == ("x", {"foo": 2, "bar": True})
 
-    assert a.build("x", {"foo": 1, "bar": False}) == prefix + "/"
-    assert a.build("x", {"foo": 2, "bar": False}) == prefix + "/2"
-    assert a.build("x", {"bar": False}) == prefix + "/"
-    assert a.build("x", {"foo": 1, "bar": True}) == prefix + "/bar/"
-    assert a.build("x", {"foo": 2, "bar": True}) == prefix + "/bar/2"
-    assert a.build("x", {"bar": True}) == prefix + "/bar/"
+    assert a.build("x", {"foo": 1, "bar": False}) == f"{prefix}/"
+    assert a.build("x", {"foo": 2, "bar": False}) == f"{prefix}/2"
+    assert a.build("x", {"bar": False}) == f"{prefix}/"
+    assert a.build("x", {"foo": 1, "bar": True}) == f"{prefix}/bar/"
+    assert a.build("x", {"foo": 2, "bar": True}) == f"{prefix}/bar/2"
+    assert a.build("x", {"bar": True}) == f"{prefix}/bar/"
 
 
 def test_building_bytes():
@@ -999,7 +1048,7 @@ def test_redirect_request_exception_code():
     exc = r.RequestRedirect("http://www.google.com/")
     exc.code = 307
     env = create_environ()
-    strict_eq(exc.get_response(env).status_code, exc.code)
+    assert exc.get_response(env).status_code == exc.code
 
 
 def test_redirect_path_quoting():
@@ -1014,38 +1063,38 @@ def test_redirect_path_quoting():
     with pytest.raises(r.RequestRedirect) as excinfo:
         adapter.match("/foo bar/page/1")
     response = excinfo.value.get_response({})
-    strict_eq(response.headers["location"], u"http://example.com/foo%20bar")
+    assert response.headers["location"] == "http://example.com/foo%20bar"
 
 
 def test_unicode_rules():
     m = r.Map(
-        [r.Rule(u"/войти/", endpoint="enter"), r.Rule(u"/foo+bar/", endpoint="foobar")]
+        [r.Rule("/войти/", endpoint="enter"), r.Rule("/foo+bar/", endpoint="foobar")]
     )
-    a = m.bind(u"☃.example.com")
+    a = m.bind("☃.example.com")
     with pytest.raises(r.RequestRedirect) as excinfo:
-        a.match(u"/войти")
-    strict_eq(
-        excinfo.value.new_url,
-        "http://xn--n3h.example.com/%D0%B2%D0%BE%D0%B9%D1%82%D0%B8/",
+        a.match("/войти")
+    assert (
+        excinfo.value.new_url
+        == "http://xn--n3h.example.com/%D0%B2%D0%BE%D0%B9%D1%82%D0%B8/"
     )
 
-    endpoint, values = a.match(u"/войти/")
-    strict_eq(endpoint, "enter")
-    strict_eq(values, {})
+    endpoint, values = a.match("/войти/")
+    assert endpoint == "enter"
+    assert values == {}
 
     with pytest.raises(r.RequestRedirect) as excinfo:
-        a.match(u"/foo+bar")
-    strict_eq(excinfo.value.new_url, "http://xn--n3h.example.com/foo+bar/")
+        a.match("/foo+bar")
+    assert excinfo.value.new_url == "http://xn--n3h.example.com/foo+bar/"
 
-    endpoint, values = a.match(u"/foo+bar/")
-    strict_eq(endpoint, "foobar")
-    strict_eq(values, {})
+    endpoint, values = a.match("/foo+bar/")
+    assert endpoint == "foobar"
+    assert values == {}
 
     url = a.build("enter", {}, force_external=True)
-    strict_eq(url, "http://xn--n3h.example.com/%D0%B2%D0%BE%D0%B9%D1%82%D0%B8/")
+    assert url == "http://xn--n3h.example.com/%D0%B2%D0%BE%D0%B9%D1%82%D0%B8/"
 
     url = a.build("foobar", {}, force_external=True)
-    strict_eq(url, "http://xn--n3h.example.com/foo+bar/")
+    assert url == "http://xn--n3h.example.com/foo+bar/"
 
 
 def test_empty_path_info():
@@ -1063,24 +1112,24 @@ def test_empty_path_info():
 
 
 def test_both_bind_and_match_path_info_are_none():
-    m = r.Map([r.Rule(u"/", endpoint="index")])
+    m = r.Map([r.Rule("/", endpoint="index")])
     ma = m.bind("example.org")
-    strict_eq(ma.match(), ("index", {}))
+    assert ma.match() == ("index", {})
 
 
 def test_map_repr():
-    m = r.Map([r.Rule(u"/wat", endpoint="enter"), r.Rule(u"/woop", endpoint="foobar")])
+    m = r.Map([r.Rule("/wat", endpoint="enter"), r.Rule("/woop", endpoint="foobar")])
     rv = repr(m)
-    strict_eq(rv, "Map([<Rule '/woop' -> foobar>, <Rule '/wat' -> enter>])")
+    assert rv == "Map([<Rule '/woop' -> foobar>, <Rule '/wat' -> enter>])"
 
 
 def test_empty_subclass_rules_with_custom_kwargs():
     class CustomRule(r.Rule):
         def __init__(self, string=None, custom=None, *args, **kwargs):
             self.custom = custom
-            super(CustomRule, self).__init__(string, *args, **kwargs)
+            super().__init__(string, *args, **kwargs)
 
-    rule1 = CustomRule(u"/foo", endpoint="bar")
+    rule1 = CustomRule("/foo", endpoint="bar")
     try:
         rule2 = rule1.empty()
         assert rule1.rule == rule2.rule
@@ -1091,9 +1140,9 @@ def test_empty_subclass_rules_with_custom_kwargs():
 def test_finding_closest_match_by_endpoint():
     m = r.Map(
         [
-            r.Rule(u"/foo/", endpoint="users.here"),
-            r.Rule(u"/wat/", endpoint="admin.users"),
-            r.Rule(u"/woop", endpoint="foo.users"),
+            r.Rule("/foo/", endpoint="users.here"),
+            r.Rule("/wat/", endpoint="admin.users"),
+            r.Rule("/woop", endpoint="foo.users"),
         ]
     )
     adapter = m.bind("example.com")
@@ -1104,18 +1153,18 @@ def test_finding_closest_match_by_endpoint():
 
 
 def test_finding_closest_match_by_values():
-    rule_id = r.Rule(u"/user/id/<id>/", endpoint="users")
-    rule_slug = r.Rule(u"/user/<slug>/", endpoint="users")
-    rule_random = r.Rule(u"/user/emails/<email>/", endpoint="users")
+    rule_id = r.Rule("/user/id/<id>/", endpoint="users")
+    rule_slug = r.Rule("/user/<slug>/", endpoint="users")
+    rule_random = r.Rule("/user/emails/<email>/", endpoint="users")
     m = r.Map([rule_id, rule_slug, rule_random])
     adapter = m.bind("example.com")
     assert r.BuildError("x", {"slug": ""}, None, adapter).suggested == rule_slug
 
 
 def test_finding_closest_match_by_method():
-    post = r.Rule(u"/post/", endpoint="foobar", methods=["POST"])
-    get = r.Rule(u"/get/", endpoint="foobar", methods=["GET"])
-    put = r.Rule(u"/put/", endpoint="foobar", methods=["PUT"])
+    post = r.Rule("/post/", endpoint="foobar", methods=["POST"])
+    get = r.Rule("/get/", endpoint="foobar", methods=["GET"])
+    put = r.Rule("/put/", endpoint="foobar", methods=["PUT"])
     m = r.Map([post, get, put])
     adapter = m.bind("example.com")
     assert r.BuildError("invalid", {}, "POST", adapter).suggested == post
@@ -1129,7 +1178,7 @@ def test_finding_closest_match_when_none_exist():
 
 
 def test_error_message_without_suggested_rule():
-    m = r.Map([r.Rule(u"/foo/", endpoint="world", methods=["GET"])])
+    m = r.Map([r.Rule("/foo/", endpoint="world", methods=["GET"])])
     adapter = m.bind("example.com")
 
     with pytest.raises(r.BuildError) as excinfo:
@@ -1150,7 +1199,7 @@ def test_error_message_without_suggested_rule():
 
 
 def test_error_message_suggestion():
-    m = r.Map([r.Rule(u"/foo/<id>/", endpoint="world", methods=["GET"])])
+    m = r.Map([r.Rule("/foo/<id>/", endpoint="world", methods=["GET"])])
     adapter = m.bind("example.com")
 
     with pytest.raises(r.BuildError) as excinfo:
